use adw::{prelude::*, subclass::prelude::*};
use gtk::{gio, glib};

use crate::{config::*, i18n::gettext, ElevadoWindow};
mod imp {
    use super::*;

    #[derive(Debug, Default)]
    pub struct ElevadoApplication;

    #[glib::object_subclass]
    impl ObjectSubclass for ElevadoApplication {
        type ParentType = adw::Application;
        type Type = super::ElevadoApplication;

        const NAME: &'static str = "ElevadoApplication";
    }

    impl ObjectImpl for ElevadoApplication {
        fn constructed(&self) {
            self.parent_constructed();
            let obj = self.obj();
            obj.setup_gactions();
            obj.set_accels_for_action("app.quit", &["<primary>q"]);
        }
    }

    impl ApplicationImpl for ElevadoApplication {
        fn activate(&self) {
            let application = self.obj();
            // Get the current window or create one if necessary
            let window = application
                .active_window()
                .unwrap_or(ElevadoWindow::new(&*application).upcast());

            if PROFILE == "development" {
                window.add_css_class("devel");
            }

            window.present();
        }
    }

    impl GtkApplicationImpl for ElevadoApplication {}
    impl AdwApplicationImpl for ElevadoApplication {}
}

glib::wrapper! {
    pub struct ElevadoApplication(ObjectSubclass<imp::ElevadoApplication>)
        @extends gio::Application, gtk::Application, adw::Application,
        @implements gio::ActionGroup, gio::ActionMap;
}

impl ElevadoApplication {
    fn setup_gactions(&self) {
        let quit_action = gio::ActionEntry::builder("quit")
            .activate(move |app: &Self, _, _| app.quit())
            .build();
        let about_action = gio::ActionEntry::builder("about")
            .activate(move |app: &Self, _, _| app.show_about())
            .build();
        self.add_action_entries([quit_action, about_action]);
    }

    fn show_about(&self) {
        let window = self
            .active_window()
            .expect("window must be active before About is opened");
        let about =
            adw::AboutDialog::from_appdata("/com/feaneron/Elevado/metainfo.xml", Some(VERSION));

        about.set_artists(&["Brage Fuglseth https://bragefuglseth.dev"]);
        about.set_comments("Accessibility inspector");
        about.set_copyright("© 2025 Georges Stavracas");
        about.set_developer_name("Georges Basile Stavracas Neto");
        about.set_developers(&[
            "Georges Basile Stavracas Neto <georges.stavracas@gmail.com>",
            "Claire (qwery) <qwerymailbox@gmail.com>",
        ]);
        about.set_version(VERSION);
        // Translators: Replace "translator-credits" with your name/username, and
        // optionally an email or URL.
        about.set_translator_credits(&gettext("translator-credits"));

        about.present(Some(&window));
    }
}

impl Default for ElevadoApplication {
    fn default() -> Self {
        glib::Object::builder()
            .property("application-id", APP_ID)
            .property("resource-base-path", "/com/feaneron/Elevado")
            .property("flags", gio::ApplicationFlags::empty())
            .build()
    }
}
