#![deny(
    clippy::pedantic,
    clippy::suspicious,
    clippy::print_stdout,
    clippy::unwrap_used
)]
#![warn(clippy::allow_attributes)]
#![allow(clippy::wildcard_imports)]

mod application;
mod config;
mod i18n;
mod model;
mod panels;
mod traits;
mod view;
mod window;

use std::sync::OnceLock;

use atspi::{events::ObjectEvents, AccessibilityConnection};
use config::*;
use gettextrs::{bind_textdomain_codeset, bindtextdomain, textdomain};
use gtk::{gio, glib, prelude::*};

use self::{application::ElevadoApplication, window::ElevadoWindow};

fn main() -> glib::ExitCode {
    // make sure the bus is created before we run the app
    let _ = bus();

    // Set up gettext translations
    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR).expect("Unable to bind the text domain");
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8")
        .expect("Unable to set the text domain encoding");
    textdomain(GETTEXT_PACKAGE).expect("Unable to switch to the text domain");

    // Load resources
    let resources = gio::Resource::load(PKGDATADIR.to_owned() + "/elevado.gresource")
        .expect("Could not load resources");
    gio::resources_register(&resources);

    let rt = ElevadoApplication::default().run();

    // Deregister all events before exiting
    smol::block_on(async {
        bus()
            .deregister_event::<ObjectEvents>()
            .await
            .expect("deregistering events shouldn't fail");
    });

    rt
}

fn bus() -> &'static AccessibilityConnection {
    static A11YBUS: OnceLock<AccessibilityConnection> = OnceLock::new();
    A11YBUS.get_or_init(|| {
        smol::block_on(async {
            AccessibilityConnection::new()
                .await
                .expect("atspi bus should be available")
        })
    })
}
