use std::cell::RefCell;

use adw::{prelude::*, subclass::prelude::*};
use glib::Properties;
use gtk::glib;

use crate::{i18n::gettext, model::ElevadoNode};

mod imp {
    use super::*;

    #[derive(Default, Properties, gtk::CompositeTemplate)]
    #[properties(wrapper_type = super::ElevadoSidebarRow)]
    #[template(resource = "/com/feaneron/Elevado/panels/sidebar_row.ui")]
    pub struct ElevadoSidebarRow {
        #[property(get, set = Self::set_node)]
        pub(super) node: RefCell<ElevadoNode>,

        #[template_child]
        pub(super) name_label: TemplateChild<gtk::Label>,
    }

    #[glib::object_subclass]
    impl ObjectSubclass for ElevadoSidebarRow {
        type ParentType = adw::Bin;
        type Type = super::ElevadoSidebarRow;

        const NAME: &'static str = "ElevadoSidebarRow";

        fn class_init(klass: &mut Self::Class) {
            klass.bind_template();
        }

        fn instance_init(obj: &glib::subclass::InitializingObject<Self>) {
            obj.init_template();
        }
    }

    #[glib::derived_properties]
    impl ObjectImpl for ElevadoSidebarRow {
        fn constructed(&self) {
            self.parent_constructed();

            self.node.replace(ElevadoNode::default());
        }
    }
    impl WidgetImpl for ElevadoSidebarRow {}
    impl BinImpl for ElevadoSidebarRow {}

    impl ElevadoSidebarRow {
        fn update_row_for_node(&self) {
            if let Some(name) = self.obj().node().name() {
                self.name_label.set_label(&name);
                return;
            }
            self.name_label.set_label(&gettext("Unnamed"));
        }

        fn set_node(&self, value: Option<ElevadoNode>) {
            if let Some(node) = value {
                self.node.replace(node);
                self.update_row_for_node();
            }
        }
    }
}

glib::wrapper! {
    pub struct ElevadoSidebarRow(ObjectSubclass<imp::ElevadoSidebarRow>)
        @extends gtk::Widget, adw::Bin,
        @implements gtk::Accessible, gtk::Buildable, gtk::ConstraintTarget;
}
