pub trait Iconic {
    /// Get an icon name to represent this data
    fn iconify(&self) -> &'static str;
}

#[expect(clippy::enum_glob_use)]
impl Iconic for atspi::Role {
    fn iconify(&self) -> &'static str {
        use atspi::Role::*;
        match self {
            Application => "application-x-executable-symbolic",
            AcceleratorLabel => "tag-outline-symbolic",
            Alert => "preferences-system-notifications-symbolic",
            Arrow => "code-symbolic",
            Calendar => "x-office-calendar-symbolic",
            Dialog => "dialog-symbolic",
            PageTab | PageTabList => "tab-symbolic",
            Table | TableCell | TableColumnHeader | TableRowHeader | Tree | TreeTable => {
                "table-symbolic"
            }
            Window => "window-symbolic",
            Menu | MenuBar | MenuItem | InfoBar => "banner-symbolic",
            Frame | DesktopFrame | InternalFrame => "window-flat-symbolic",
            CheckMenuItem | CheckBox => "checkbox-checked-symbolic",
            RadioButton | RadioMenuItem => "radio-checked-symbolic",
            Separator => "view-continuous-symbolic",
            SplitPane | Panel => "panel-left-symbolic",
            ScrollBar | ScrollPane => "window-scrolling-symbolic",
            Filler => "selection-opaque-2-symbolic",
            List | ListItem => "list-symbolic",
            DocumentSpreadsheet => "x-office-spreadsheet-symbolic",
            DocumentPresentation => "x-office-presentation-symbolic",
            DocumentText | Text | FontChooser => "font-x-generic-symbolic",
            DocumentEmail => "mail-unread-symbolic",
            DocumentWeb => "web-browser-symbolic",
            Link => "chain-link-symbolic",
            Icon | Image => "image-x-generic-symbolic",
            ColorChooser => "color-picker-symbolic",
            Dial => "dialpad-symbolic",
            FileChooser => "file-manager-symbolic",
            PasswordText => "password-entry-symbolic",
            Audio => "music-note-symbolic",
            Video => "video-clip-symbolic",
            Terminal => "terminal-symbolic",
            Timer => "timer-symbolic",
            _ => "dialog-question-symbolic",
        }
    }
}
