use crate::i18n::gettext;

pub trait Translatable {
    /// Get the localized name of the data type
    fn translate(&self) -> String;
}

impl Translatable for atspi::Role {
    fn translate(&self) -> String {
        gettext(self.name())
    }
}

impl Translatable for atspi::State {
    fn translate(&self) -> String {
        gettext(self.to_static_str())
    }
}

impl Translatable for atspi::RelationType {
    fn translate(&self) -> String {
        match self {
            Self::Null => String::new(),
            Self::LabelFor => gettext("Label For"),
            Self::LabelledBy => gettext("Labelled By"),
            Self::ControllerFor => gettext("Controller For"),
            Self::ControlledBy => gettext("Controlled By"),
            Self::MemberOf => gettext("Member Of"),
            Self::TooltipFor => gettext("Tooltip For"),
            Self::NodeChildOf => gettext("Node Child Of"),
            Self::NodeParentOf => gettext("Node Parent Of"),
            Self::Extended => gettext("Extended"),
            Self::FlowsTo => gettext("Flows To"),
            Self::FlowsFrom => gettext("Flows From"),
            Self::SubwindowOf => gettext("Subwindow Of"),
            Self::Embeds => gettext("Embeds"),
            Self::EmbeddedBy => gettext("Embedded By"),
            Self::PopupFor => gettext("Popup For"),
            Self::ParentWindowOf => gettext("Parent Window Of"),
            Self::DescriptionFor => gettext("Description For"),
            Self::DescribedBy => gettext("Described By"),
            Self::Details => gettext("Details"),
            Self::DetailsFor => gettext("Details For"),
            Self::ErrorMessage => gettext("Error Message"),
            Self::ErrorFor => gettext("Error For"),
        }
    }
}

impl Translatable for atspi::Interface {
    fn translate(&self) -> String {
        match self {
            Self::Accessible => gettext("Accessible"),
            Self::Action => gettext("Action"),
            Self::Application => gettext("Application"),
            Self::Cache => gettext("Cache"),
            Self::Collection => gettext("Collection"),
            Self::Component => gettext("Component"),
            Self::Document => gettext("Document"),
            Self::DeviceEventController => gettext("Device Event Controller"),
            Self::DeviceEventListener => gettext("Device Event Listener"),
            Self::EditableText => gettext("Editable Text"),
            Self::Hyperlink => gettext("Hyperlink"),
            Self::Hypertext => gettext("Hypertext"),
            Self::Image => gettext("Image"),
            Self::Registry => gettext("Registry"),
            Self::Selection => gettext("Selection"),
            Self::Socket => gettext("Socket"),
            Self::Table => gettext("Table"),
            Self::TableCell => gettext("TableCell"),
            Self::Text => gettext("Text"),
            Self::Value => gettext("Value"),
        }
    }
}

impl Translatable for atspi::Layer {
    fn translate(&self) -> String {
        match self {
            atspi::Layer::Invalid => gettext("Invalid"),
            atspi::Layer::Background => gettext("Background"),
            atspi::Layer::Canvas => gettext("Canvas"),
            atspi::Layer::Widget => gettext("Widget"),
            atspi::Layer::Mdi => gettext("MDI"),
            atspi::Layer::Popup => gettext("Popup"),
            atspi::Layer::Overlay => gettext("Overlay"),
            atspi::Layer::Window => gettext("Window"),
        }
    }
}
