use std::cell::RefCell;

use adw::{prelude::*, subclass::prelude::*};
use glib::Properties;
use gtk::glib;

use crate::{i18n::gettext, model::ElevadoNode};

mod imp {
    use super::*;

    #[derive(Debug, Default, Properties, gtk::CompositeTemplate)]
    #[properties(wrapper_type = super::ElevadoNodeRow)]
    #[template(resource = "/com/feaneron/Elevado/view/node_row.ui")]
    pub struct ElevadoNodeRow {
        #[property(get, set)]
        pub(super) row: RefCell<Option<gtk::TreeListRow>>,

        #[template_child]
        pub(super) expander: TemplateChild<gtk::TreeExpander>,
    }

    #[glib::object_subclass]
    impl ObjectSubclass for ElevadoNodeRow {
        type ParentType = adw::Bin;
        type Type = super::ElevadoNodeRow;

        const NAME: &'static str = "ElevadoNodeRow";

        fn class_init(klass: &mut Self::Class) {
            klass.bind_template();
            klass.bind_template_callbacks();
        }

        fn instance_init(obj: &glib::subclass::InitializingObject<Self>) {
            obj.init_template();
        }
    }

    #[glib::derived_properties]
    impl ObjectImpl for ElevadoNodeRow {}
    impl WidgetImpl for ElevadoNodeRow {}
    impl BinImpl for ElevadoNodeRow {}

    #[gtk::template_callbacks]
    impl ElevadoNodeRow {
        #[expect(clippy::unused_self)]
        #[template_callback]
        fn accessible_title_from_node_name_cb(&self, name: Option<String>) -> String {
            if let Some(obj_title) = name {
                obj_title
            } else {
                gettext("Unnamed Object")
            }
        }

        #[expect(clippy::unused_self)]
        #[template_callback]
        fn css_classes_from_node_name_cb(&self, value: Option<ElevadoNode>) -> Vec<String> {
            use atspi::{Role, State};

            let mut css_classes: Vec<String> = vec![];
            if let Some(node) = value {
                let state = node.state();
                let role = node.role().unwrap_or(Role::Invalid);

                if (!state.contains(State::Visible)) && (role != Role::Application) {
                    css_classes.push("dim-label".to_string());
                }
            }
            css_classes
        }
    }
}

glib::wrapper! {
    pub struct ElevadoNodeRow(ObjectSubclass<imp::ElevadoNodeRow>)
        @extends gtk::Widget, adw::Bin,
        @implements gtk::Accessible, gtk::Buildable, gtk::ConstraintTarget;
}

impl ElevadoNodeRow {
    pub fn collapse(&self) {
        self.imp()
            .expander
            .activate_action("listitem.collapse", None)
            .expect("listitem.collapse action should exist");
    }

    pub fn expand(&self) {
        self.imp()
            .expander
            .activate_action("listitem.expand", None)
            .expect("listitem.expand action should exist");
    }
}
